%{

#include <qmake_yacc.h>
#include <stdlib.h>

/*
%option debug
*/
%}

%option noyywrap

%x list
%x funcargs

delim     [ \t]
ws        {delim}+
letter    [A-Za-z]
digit     [0-9]
id_simple ({digit}|{letter}|\!|-|_|\*|\$)({letter}|{digit}|\||\!|-|_|\*|\$|\.|\+|\-)*
id_list   [^\n]*\\{ws}*
id_cmd    [^\n]*
id_args   [^\n]*\)
number    {digit}+
comment   #.*
comment_cont   {ws}*#.*\n
id_list_single   [^\n]*
cont      \\{ws}*\n

%%

<list,INITIAL>{ws} {}
<list,INITIAL>{cont}   { BEGIN(list); return CONT; }
{id_simple} { yylval.value = yytext; return (ID_SIMPLE); }

<list>{id_list} {
    yylval.value = yytext;
    yylval.value = yylval.value.mid(0, yylval.value.findRev("\\")-1);
    unput('\\');
    BEGIN(INITIAL);
    return (ID_LIST); 
    }

<list>{comment_cont} { yylval.value = yytext; BEGIN(list); return (LIST_COMMENT); }
<list>{id_list_single} {
    yylval.value = yytext;
    BEGIN(INITIAL);
    return (ID_LIST_SINGLE); 
    }
    
<funcargs>{id_args} { 
    yylval.value = yytext;
    yylval.value = yylval.value.mid(0, yylval.value.length()-1);
    unput(')');
    BEGIN(INITIAL);
    return (ID_ARGS); 
    }

"="      { BEGIN(list); yylval.value = yytext; return EQ; }
"+="     { BEGIN(list); yylval.value = yytext; return PLUSEQ; }
"-="     { BEGIN(list); yylval.value = yytext; return MINUSQE; }
"*="     { BEGIN(list); yylval.value = yytext; return STAREQ; }
"~="     { BEGIN(list); yylval.value = yytext; return TILDEEQ; }
"{"      { return LCURLY; }
"}"      { return RCURLY; }
"("      { BEGIN(funcargs); return LBRACE; }
<funcargs,INITIAL>")"      { BEGIN(INITIAL); return RBRACE; }
"!"      { return NOT; }
":"      { yylval.value = yytext; return COLON; }
"."      { return DOT; }
<list,INITIAL>"\n" { BEGIN(INITIAL); return NEWLINE; }
{comment} { yylval.value = yytext; return (COMMENT); }

%%

/*
"#"      { return NUMSIGN; }
<custom_list>{id_ex}  { yylval.value = yytext; BEGIN(INITIAL); return (ID_EX); }
{number} { yylval.int_val = atoi(yytext); return (NUMBER); }
*/
