
%{

/**
@todo Recognize comments after statements like:
SOURCES = foo #regognize me

@fixme Parser fails on files that do not end with a newline
@fixme 1 shift/reduce conflict in "line_body" rule
*/

#include "qmakeast.h"

#define YYSTYPE_IS_DECLARED

using namespace QMake;

struct Result {
    Result(): node(0) {}

    QString value;
    AST *node;
    QStringList values;
};

//typedef QString YYSTYPE;
typedef Result YYSTYPE;

void yyerror(const char *str) {
    printf("%s", str);
}

int yylex();

ProjectAST *projectAST;

/*
%debug
*/
%}

%token ID_SIMPLE
%token ID_LIST
%token LBRACE
%token EQ
%token PLUSEQ
%token MINUSQE
%token STAREQ
%token TILDEEQ
%token LBRACE
%token RBRACE
%token NOT
%token COLON
%token NUMSIGN
%token NEWLINE
%token NUMBER
%token COMMENT
%token CONT
%token DOT
%token ID_EX
%token RCURLY
%token LCURLY
%token ID_ARGS
%token LIST_COMMENT
%token ID_LIST_SINGLE

%%

project : 
    {
        if (projectAST)
            delete projectAST;
        projectAST = 0;
    }
    statements
    ;
    
statements : statements statement
        {
            if (!projectAST)
                projectAST = new ProjectAST();
            projectAST->addChildAST($<node>2);
        }
    |
    ;

statement : variable_assignment 
        {
            $<node>$ = $<node>1;
        }
    | scope 
        {
            $<node>$ = 0;
        }
    | function_call
        {
            $<node>$ = 0;
        }
    | comment
        {
            $<node>$ = 0;
        }
    | emptyline
        {
            $<node>$ = new NewLineAST();
        }
    ;

variable_assignment : scoped_identifier operator multiline_values
        {
            AssignmentAST *node = new AssignmentAST();
            node->scopedID = $<value>1;
            node->op = $<value>2;
            node->values = $<values>3;
            $<node>$ = node;
        }
    ;

scoped_identifier : ID_SIMPLE COLON scoped_identifier  { $<value>$ = $<value>1 + $<value>2 + $<value>2; }
    | ID_SIMPLE COLON  { $<value>$ = $<value>1 + $<value>2; }
    | ID_SIMPLE
    ;

multiline_values : multiline_values line_body
        {
            $<values>$.append($<value>2);
        }
    |   { $<values>$.clear(); }
    ;

line_body : ID_LIST CONT         { $<value>$ = $<value>1 + " \\\n"; }
    | ID_LIST_SINGLE NEWLINE     { $<value>$ = $<value>1 + "\n"; }
    | CONT                       { $<value>$ = "\\\n"; }
    | LIST_COMMENT
    | RBRACE                     { $<value>$ = ""; }
    ;


operator : EQ | PLUSEQ | MINUSQE | STAREQ | TILDEEQ
    ;
    
scope : scoped_identifier scope_body
    ;

function_call : scoped_identifier LBRACE function_args RBRACE scope_body else_statement
    | scoped_identifier LBRACE function_args RBRACE COLON variable_assignment
    ;
    
function_args : ID_ARGS
    |
    ;

scope_body : LCURLY statements RCURLY
    |
    ;
    
else_statement : "else" LCURLY statements RCURLY
    | "else" COLON variable_assignment
    |
    ;

comment : COMMENT NEWLINE
    ;

emptyline : NEWLINE
    ;
    
%%

#include "qmake_lex.cpp"
