/***************************************************************************
 *   Copyright (C) 2005 by Alexander Dymo                                  *
 *   adymo@kdevelop.org                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef QMAKEQMAKEAST_H
#define QMAKEQMAKEAST_H

#include <qstringlist.h>

namespace QMake {

class AST {
public:
    enum NodeType {
        ProjectAST,
        AssignmentAST,
        NewLineAST
    };
    
    AST(NodeType nodeType): m_nodeType(nodeType) {}
    virtual ~AST();
    
    virtual void addChildAST(AST *node);
    virtual void writeBack(QString &buffer);
    
    NodeType nodeType() const { return m_nodeType; }
    
protected:
    NodeType m_nodeType;
    QValueList<AST*> m_children;

};

class StatementAST;

class ProjectAST: public AST {
public:
    ProjectAST(): AST(AST::ProjectAST) {}
    
    virtual void addChildAST(AST *node);

    QValueList<QMake::StatementAST*> statements;    
    
};

class StatementAST: public AST {
public:
    StatementAST(NodeType nodeType): AST(nodeType) {}

};

class AssignmentAST: public StatementAST {
public:
    AssignmentAST(): StatementAST(AST::AssignmentAST) {}

    virtual void writeBack(QString &buffer);

    QString scopedID;
    QString op;
    QStringList values;
};

class NewLineAST: public AST {
public:
    NewLineAST(): AST(AST::NewLineAST) {}
    
    virtual void writeBack(QString &buffer);

};

}

#endif
